package com.example.demo;

import static org.junit.jupiter.api.Assertions.assertEquals;

import jakarta.persistence.EntityManager;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.transaction.annotation.Transactional;

@SpringBootTest
public class AnimalRepositoryTest {

    @Autowired
    private AnimalRepository animalRepository;
    @Autowired
    private EntityManager entityManager;

    @BeforeEach
    void setUp() {
        animalRepository.deleteAll();
        animalRepository.save(new Cat("Cat"));
        animalRepository.save(new Dog("Dog"));
    }

    @Test
    void countDogsShouldFindOneDog() {
        var dogs = animalRepository.countDogs();
        assertEquals(1, dogs);
        dogs = animalRepository.countDogs();
        assertEquals(1, dogs);
    }

    @Test
    void countDogsWithTypeShouldFindOneDog() {
        var dogs = animalRepository.count(Dog.class);
        assertEquals(1, dogs);
        dogs = animalRepository.count(Dog.class);
        assertEquals(1, dogs);
    }

    @Test
    void countDogsWithQueryDslShouldFindOneDog() {
        var q = QAnimal.animal;
        var expression = q.instanceOf(Dog.class);
        var dogs = animalRepository.count(expression);
        assertEquals(1L, dogs);
        dogs = animalRepository.count(expression);
        assertEquals(1L, dogs);
    }

    @Test
    void countDogsWithJpaMethodShouldFindOneDog() {
        var dogs = animalRepository.countWithJpa(Dog.class);
        assertEquals(1L, dogs);
        dogs = animalRepository.countWithJpa(Dog.class);
        assertEquals(1L, dogs);
    }

    @Test
    void countDogsWithJpaShouldFindOneDog() {
        var criteriaBuilder = entityManager.getCriteriaBuilder();
        var criteria = criteriaBuilder.createQuery(Long.class);
        var root = criteria.from(Animal.class);
        criteria.select(criteriaBuilder.count(root));
        criteria.where(criteriaBuilder.equal(root.type(), Dog.class));
        var dogs = entityManager.createQuery(criteria).getSingleResult();
        assertEquals(1L, dogs);
        dogs = entityManager.createQuery(criteria).getSingleResult();
        assertEquals(1L, dogs);
    }

    @Test
    void countDogsWithJpqlShouldFindOneDog() {
        var query = entityManager.createQuery("select count(animal) from Animal animal where type(animal) = Dog");
        var dogs = query.getSingleResult();
        assertEquals(1L, dogs);
        query = entityManager.createQuery("select count(animal) from Animal animal where type(animal) = Dog");
        dogs = query.getSingleResult();
        assertEquals(1L, dogs);
    }

    @Test
    void countDogsWithJpqlAndParamShouldFindOneDog() {
        var query = entityManager.createQuery("select count(animal) from Animal animal where type(animal) = ?1");
        query.setParameter(1, Dog.class);
        var dogs = query.getSingleResult();
        assertEquals(1L, dogs);
        query = entityManager.createQuery("select count(animal) from Animal animal where type(animal) = ?1");
        query.setParameter(1, Dog.class);
        dogs = query.getSingleResult();
        assertEquals(1L, dogs);
    }

    @Test
    void countDogsWithJpqlAndNamedParamShouldFindOneDog() {
        var query = entityManager.createQuery("select count(animal) from Animal animal where type(animal) = :animal");
        query.setParameter("animal", Dog.class);
        var dogs = query.getSingleResult();
        assertEquals(1L, dogs);
        query = entityManager.createQuery("select count(animal) from Animal animal where type(animal) = :animal");
        dogs = query.getSingleResult();
        query.setParameter("animal", Dog.class);
        assertEquals(1L, dogs);
    }
}
